#ifndef OS_H
#define OS_H

#ifdef _KERNEL

/* Here is all the os-dependend structures definitions */

/* Handle some "struct proc" -> "struct lwp" issues */
#if defined (__NetBSD__) && (__NetBSD_Version__ < 399001400)
# define ZPROC	struct proc
#else
# define ZPROC	struct lwp
#endif

#if defined(__OpenBSD__) || defined(__NetBSD__)
#include <sys/exec.h>
#include <sys/lkm.h>
#ifdef __NetBSD__
#define ovbcopy(src, dst, len)		memmove((dst), (src), (len))
#include <sys/stat.h>
#endif
#endif

#if defined(__FreeBSD__) || defined(__NetBSD__)
#define timer_t struct callout
#else
#define timer_t struct timeout 
#endif

/* Locking stuff */
#ifdef __FreeBSD__
#	if __FreeBSD_version < 500000
#		define lock_t	struct lock
#		define ZAP_LOCK(_lock) simple_lock(&(_lock))
#		define ZAP_UNLOCK(_lock) simple_unlock(&(_lock))
#		define ZAP_LOCK_INIT(_lock, name) simple_lock_init(&(_lock))
#		define ZAP_LOCK_DESTROY(_lock) do{}while(0)
	/* lockmgr locks */
#		define	ZAP_MGR_LOCK(_lock, flags) lockmgr(&(_lock), \
			flags, 0, curproc)
#		define	ZAP_MGR_DESTROY_LOCK(_lock) do{}while(0)
#	else
#		define lock_t	struct mtx
#		define ZAP_LOCK(_lock)           	mtx_lock(&(_lock))
#		define ZAP_UNLOCK(_lock)         	mtx_unlock(&(_lock))
#		define ZAP_LOCK_INIT(_lock, name)	mtx_init(&(_lock), \
							name, NULL, MTX_DEF)
#		define ZAP_LOCK_DESTROY(_lock)		mtx_destroy(&(_lock))
		/* lockmgr locks */
#		define	ZAP_MGR_LOCK(_lock, flags) lockmgr(&(_lock), \
							flags, 0, curthread); 
#		define	ZAP_MGR_DESTROY_LOCK(_lock) lockdestroy(&(_lock))
#	endif
#else  /* NetBSD, OpenBSD */
#	define lock_t	struct simplelock
#	define ZAP_LOCK(_lock) simple_lock(&(_lock))
#	define ZAP_UNLOCK(_lock) simple_unlock(&(_lock))
#	define ZAP_LOCK_INIT(_lock, name) simple_lock_init(&(_lock))
#	define ZAP_LOCK_DESTROY(_lock) do{}while(0)
	/* lockmgr locks */
#ifdef __NetBSD__
#		define	ZAP_MGR_LOCK(_lock, flags) lockmgr(&(_lock), \
			flags, 0)
#else  /* OpenBSD */
#	define	ZAP_MGR_LOCK(_lock, flags) lockmgr(&(_lock), \
		flags, 0, curproc)
#endif
#	define	ZAP_MGR_DESTROY_LOCK(_lock) do{}while(0)
#endif

/* fileops */
#ifdef __FreeBSD__
#if __FreeBSD_version < 500000
typedef int fo_rdwr_t(struct file *fp, struct uio *uio,
			struct ucred *cred, int flags,
			struct proc *p);
typedef int fo_ioctl_t(struct file *fp, u_long com,
			caddr_t data, struct proc *p);
typedef int fo_poll_t(struct file *fp, int events,
			struct ucred *cred, struct proc *p);
typedef int fo_kqfilter_t(struct file *fp, struct knote *kn);
typedef int fo_stat_t(struct file *fp, struct stat *sb,
			struct proc *p);
typedef int fo_close_t(struct file *fp, struct proc *p);
#	endif
#else /* OpenBSD, NetBSD */
#ifdef __OpenBSD__
typedef int     fo_rdwr_t(struct file *, off_t *, struct uio *, struct ucred *);
typedef int     fo_ioctl_t(struct file *, u_long, caddr_t, struct proc *p);
#endif
#ifdef __NetBSD__
typedef int     fo_rdwr_t(struct file *, off_t *, struct uio *,
				struct ucred *, int);
typedef int     fo_ioctl_t(struct file *, u_long, void *, ZPROC *p);
typedef int	fo_fcntl_t(struct file *, u_int, void *, ZPROC *p);
#endif
typedef int     fo_poll_t(struct file *, int, ZPROC *);
typedef int     fo_kqfilter_t(struct file *, struct knote *);
typedef int     fo_stat_t(struct file *, struct stat *, ZPROC *);
typedef int     fo_close_t(struct file *, ZPROC *);
#endif /* def __FreeBSD__ */

/* Module stuff */
#if defined(__OpenBSD__) || defined(__NetBSD__)
#define MOD_LOAD 		LKM_E_LOAD
#define MOD_UNLOAD		LKM_E_UNLOAD
#endif

/* newer NetBSD has pseudo-returns to avoid overloading ENXIO */
#if defined (__NetBSD__) && (__NetBSD_Version__ >= 299001600)
#define EZTCLONE EMOVEFD
#else
#define EZTCLONE ENXIO
#endif

/* NetBSD has changed some of the args of sysctl_createv() to be const */
#if defined (__NetBSD__) && (__NetBSD_Version__ < 399000600)
# define __SYSCTL_CONST
#else
# define __SYSCTL_CONST const
#endif

#endif /* _KERNEL */
#endif /* OS_H */
